import os, shutil

i = 1

for f in os.listdir('/home/jason/Downloads/sopcast-translations'):
	if len(f) > 5:
		shutil.copyfile('/home/jason/Downloads/sopcast-translations/%s' % (f), '/home/jason/Downloads/sopcast-translations/fixed/%s' % (f[15:]))

for f in os.listdir('/home/jason/Downloads/sopcast-translations/fixed'):
	print "%d: %s" % (i, f)
	i = i + 1
	
	for line in open('/home/jason/Downloads/sopcast-translations/fixed/%s' % (f)):
		if "Last-Translator" in line:
			print line
